@@ -9,54 +9,47 @@
 #' user-defined model
 #'
 #' This is an experimental function and only recommended for advanced users.
-#' The selected model is fitted with lagged values of \code{y} as
-#' inputs. The inputs are for
-#' lags 1 to \code{p}, and lags \code{m} to \code{mP} where
-#' \code{m=frequency(y)}. If \code{xreg} is provided, its columns are also
-#' used as inputs. If there are missing values in \code{y} or
-#' \code{xreg}, the corresponding rows (and any others which depend on them as
-#' lags) are omitted from the fit. The model is trained for one-step
-#' forecasting. Multi-step forecasts are computed recursively.
+#' The selected model is fitted with lagged values of `y as inputs. The inputs
+#' are for lags 1 to `p`, and lags `m` to `mP` where `m = frequency(y)`. If
+#' `xreg` is provided, its columns are also used as inputs. If there are
+#' missing values in `y` or `xreg`, the corresponding rows (and any others
+#' which depend on them as lags) are omitted from the fit. The model is trained
+#' for one-step forecasting. Multi-step forecasts are computed recursively.
 #'
 #' @aliases print.modelAR
 #'
-#' @param y A numeric vector or time series of class \code{ts}.
+#' @inheritParams nnetar
 #' @param p Embedding dimension for non-seasonal time series. Number of
 #' non-seasonal lags used as inputs. For non-seasonal time series, the default
 #' is the optimal number of lags (according to the AIC) for a linear AR(p)
 #' model. For seasonal time series, the same method is used but applied to
 #' seasonally adjusted data (from an stl decomposition).
 #' @param P Number of seasonal lags used as inputs.
-#' @param FUN Function used for model fitting. Must accept argument \code{x}
-#' and \code{y} for the predictors and response, respectively (\code{formula}
-#' object not currently supported).
-#' @param predict.FUN Prediction function used to apply \code{FUN} to new data.
-#' Must accept an object of class \code{FUN} as its first argument, and a
+#' @param FUN Function used for model fitting. Must accept argument `x` and `y`
+#' for the predictors and response, respectively (`formula` object not
+#' currently supported).
+#' @param predict.FUN Prediction function used to apply `FUN` to new data.
+#' Must accept an object of class `FUN` as its first argument, and a
 #' data frame or matrix of new data for its second argument. Additionally,
 #' it should return fitted values when new data is omitted.
-#' @param xreg Optionally, a vector or matrix of external regressors, which
-#' must have the same number of rows as \code{y}. Must be numeric.
-#' @param model Output from a previous call to \code{nnetar}. If model is
-#' passed, this same model is fitted to \code{y} without re-estimating any
+#' @param model Output from a previous call to `nnetar`. If model is
+#' passed, this same model is fitted to `y` without re-estimating any
 #' parameters.
 #' @param subset Optional vector specifying a subset of observations to be used
 #' in the fit. Can be an integer index vector or a logical vector the same
-#' length as \code{y}. All observations are used by default.
-#' @param scale.inputs If TRUE, inputs are scaled by subtracting the column
-#' means and dividing by their respective standard deviations. If \code{lambda}
-#' is not \code{NULL}, scaling is applied after Box-Cox transformation.
-#' @param x Deprecated. Included for backwards compatibility.
-#' @param \dots Other arguments passed to \code{FUN} for
-#' \code{modelAR}.
-#' @inheritParams forecast.ts
+#' length as `y`. All observations are used by default.
+#' @param scale.inputs If `TRUE`, inputs are scaled by subtracting the column
+#' means and dividing by their respective standard deviations. If `lambda`
+#' is not `NULL`, scaling is applied after Box-Cox transformation.
+#' @param ... Other arguments passed to `FUN` for `modelAR`.
 #'
-#' @return Returns an object of class "\code{modelAR}".
+#' @return Returns an object of class `modelAR`.
 #'
-#' The function \code{summary} is used to obtain and print a summary of the
+#' The function `summary` is used to obtain and print a summary of the
 #' results.
 #'
-#' The generic accessor functions \code{fitted.values} and \code{residuals}
-#' extract useful features of the value returned by \code{nnetar}.
+#' The generic accessor functions `fitted.values` and `residuals`
+#' extract useful features of the value returned by `modelAR`.
 #'
 #' \item{model}{A list containing information about the fitted model}
 #' \item{method}{The name of the forecasting method as a character string}
@@ -68,11 +61,48 @@
 #'
 #' @author Rob J Hyndman and Gabriel Caceres
 #' @keywords ts
-#'
+#' @examples
+#' ## Set up functions
+#' my_lm <- function(x, y) {
+#'  structure(lsfit(x,y), class = "lsfit")
+#' }
+#' predict.lsfit <- function(object, newdata = NULL) {
+#'   n <- length(object$qr$qt)
+#'   if(is.null(newdata)) {
+#'     z <- numeric(n)
+#'     z[seq_len(object$qr$rank)] <- object$qr$qt[seq_len(object$qr$rank)]
+#'     as.numeric(qr.qy(object$qr, z))
+#'   } else {
+#'     sum(object$coefficients * c(1, newdata))
+#'   }
+#' }
+#' # Fit an AR(2) model
+#' fit <- modelAR(
+#'   y = lynx,
+#'   p = 2,
+#'   FUN = my_lm,
+#'   predict.FUN = predict.lsfit,
+#'   lambda = 0.5,
+#'   scale.inputs = TRUE
+#' )
+#' forecast(fit, h = 20) |> autoplot()
 #' @export
-modelAR <- function(y, p, P=1, FUN, predict.FUN, xreg=NULL, lambda=NULL, model=NULL, subset=NULL, scale.inputs=FALSE, x=y, ...) {
+modelAR <- function(
+  y,
+  p,
+  P = 1,
+  FUN,
+  predict.FUN,
+  xreg = NULL,
+  lambda = NULL,
+  model = NULL,
+  subset = NULL,
+  scale.inputs = FALSE,
+  x = y,
+  ...
+) {
   useoldmodel <- FALSE
-  yname <- deparse(substitute(y))
+  yname <- deparse1(substitute(y))
   if (!is.null(model)) {
     # Use previously fitted model
     useoldmodel <- TRUE
@@ -85,10 +115,17 @@ modelAR <- function(y, p, P=1, FUN, predict.FUN, xreg=NULL, lambda=NULL, model=N
     m <- max(round(frequency(model$x)), 1L)
     minlength <- max(c(model$p, model$P * m)) + 1
     if (length(x) < minlength) {
-      stop(paste("Series must be at least of length", minlength, "to use fitted model"))
+      stop(paste(
+        "Series must be at least of length",
+        minlength,
+        "to use fitted model"
+      ))
     }
     if (tsp(as.ts(x))[3] != m) {
-      warning(paste("Data frequency doesn't match fitted model, coercing to frequency =", m))
+      warning(paste(
+        "Data frequency doesn't match fitted model, coercing to frequency =",
+        m
+      ))
       x <- ts(x, frequency = m)
     }
     # Check xreg
@@ -114,23 +151,28 @@ modelAR <- function(y, p, P=1, FUN, predict.FUN, xreg=NULL, lambda=NULL, model=N
     if (!is.null(model$scalex)) {
       scale.inputs <- TRUE
     }
-  } else {                 # when not using an old model
+  } else {
+    # when not using an old model
     if (length(y) < 3) {
       stop("Not enough data to fit a model")
     }
     # Check for constant data in time series
     constant_data <- is.constant(na.interp(x))
-    if (constant_data){
-      warning("Constant data, setting p=1, P=0, lambda=NULL, scale.inputs=FALSE")
+    if (constant_data) {
+      warning(
+        "Constant data, setting p=1, P=0, lambda=NULL, scale.inputs=FALSE"
+      )
       scale.inputs <- FALSE
       lambda <- NULL
       p <- 1
       P <- 0
     }
     ## Check for constant data in xreg
-    if (!is.null(xreg)){
-      constant_xreg <- any(apply(as.matrix(xreg), 2, function(x) is.constant(na.interp(x))))
-      if (constant_xreg){
+    if (!is.null(xreg)) {
+      constant_xreg <- any(apply(as.matrix(xreg), 2, function(x) {
+        is.constant(na.interp(x))
+      }))
+      if (constant_xreg) {
         warning("Constant xreg column, setting scale.inputs=FALSE")
         scale.inputs <- FALSE
       }
@@ -138,7 +180,7 @@ modelAR <- function(y, p, P=1, FUN, predict.FUN, xreg=NULL, lambda=NULL, model=N
   }
 
   # Check for NAs in x
-  if (any(is.na(x))) {
+  if (anyNA(x)) {
     warning("Missing values in x, omitting rows")
   }
 
@@ -162,8 +204,7 @@ modelAR <- function(y, p, P=1, FUN, predict.FUN, xreg=NULL, lambda=NULL, model=N
   if (scale.inputs) {
     if (useoldmodel) {
       scalex <- model$scalex
-    }
-    else {
+    } else {
       tmpx <- scale(xx[xsub], center = TRUE, scale = TRUE)
       scalex <- list(
         center = attr(tmpx, "scaled:center"),
@@ -182,15 +223,14 @@ modelAR <- function(y, p, P=1, FUN, predict.FUN, xreg=NULL, lambda=NULL, model=N
       stop("Number of rows in xreg does not match series length")
     }
     # Check for NAs in xreg
-    if (any(is.na(xreg))) {
+    if (anyNA(xreg)) {
       warning("Missing values in xreg, omitting rows")
     }
     # Scale xreg
     if (scale.inputs) {
       if (useoldmodel) {
         scalexreg <- model$scalexreg
-      }
-      else {
+      } else {
         tmpx <- scale(xxreg[xsub, ], center = TRUE, scale = TRUE)
         scalexreg <- list(
           center = attr(tmpx, "scaled:center"),
@@ -246,7 +286,7 @@ modelAR <- function(y, p, P=1, FUN, predict.FUN, xreg=NULL, lambda=NULL, model=N
   nlag <- length(lags)
   y <- xx[-(1:maxlag)]
   lags.X <- matrix(NA_real_, ncol = nlag, nrow = n - maxlag)
-  for (i in 1:nlag){
+  for (i in 1:nlag) {
     lags.X[, i] <- xx[(maxlag - lags[i] + 1):(n - lags[i])]
   }
   # Add xreg into lagged matrix
@@ -256,14 +296,14 @@ modelAR <- function(y, p, P=1, FUN, predict.FUN, xreg=NULL, lambda=NULL, model=N
   ## Remove values not in subset
   j <- j & xsub[-(1:maxlag)]
   ## Stop if there's no data to fit (e.g. due to NAs or NaNs)
-  if (NROW(lags.X[j,, drop=FALSE]) == 0) {
+  if (NROW(lags.X[j, , drop = FALSE]) == 0) {
     stop("No data to fit (possibly due to NA or NaN)")
   }
   ## Fit selected model
   if (useoldmodel) {
     fit <- model$model
   } else {
-    fit <- FUN(x = lags.X[j,, drop=FALSE], y = y[j], ...)
+    fit <- FUN(x = lags.X[j, , drop = FALSE], y = y[j], ...)
   }
   # Return results
   out <- list()
@@ -277,15 +317,18 @@ modelAR <- function(y, p, P=1, FUN, predict.FUN, xreg=NULL, lambda=NULL, model=N
   out$scalexreg <- scalexreg
   out$xreg <- xreg
   out$lambda <- lambda
-  out$subset <- (1:length(x))[xsub]
+  out$subset <- seq_along(x)[xsub]
   out$model <- fit
   out$modelargs <- list(...)
+  fits <- rep(NA_real_, n)
+  nonmiss <- c(rep(FALSE, maxlag), j)
   if (useoldmodel) {
     out$modelargs <- model$modelargs
-    fits <- c(rep(NA_real_, maxlag), predict.FUN(fit, lags.X[j,, drop=FALSE]))
+    fits[nonmiss] <- predict.FUN(fit, lags.X[j, , drop = FALSE])
   } else {
-    fits <- c(rep(NA_real_, maxlag), predict.FUN(fit))
+    fits[nonmiss] <- predict.FUN(fit)
   }
+  out$residuals <- xx - fits
   if (scale.inputs) {
     fits <- fits * scalex$scale + scalex$center
   }
@@ -293,19 +336,21 @@ modelAR <- function(y, p, P=1, FUN, predict.FUN, xreg=NULL, lambda=NULL, model=N
   if (!is.null(lambda)) {
     fits <- InvBoxCox(fits, lambda)
   }
-  out$fitted <- ts(rep(NA_real_, length(out$x)))
-  out$fitted[c(rep(TRUE, maxlag), j)] <- fits
+  out$fitted <- ts(fits)
   tsp(out$fitted) <- tsp(out$x)
-  out$residuals <- out$x - out$fitted
   out$lags <- lags
   out$series <- yname
-  out$method <- deparse(substitute(FUN))
+  out$method <- deparse1(substitute(FUN))
   out$method <- paste0(out$method, "-AR(", p)
-  if (P > 0) out$method <- paste(out$method, ",", P, sep = "")
+  if (P > 0) {
+    out$method <- paste0(out$method, ",", P)
+  }
   out$method <- paste0(out$method, ")")
-  if (P > 0) out$method <- paste(out$method, "[", m, "]", sep = "")
+  if (P > 0) {
+    out$method <- paste0(out$method, "[", m, "]")
+  }
   out$call <- match.call()
-  return(structure(out, class = c("modelAR")))
+  structure(out, class = c("fc_model", "modelAR"))
 }
 
 #' Forecasting using user-defined model
@@ -318,79 +363,42 @@ modelAR <- function(y, p, P=1, FUN, predict.FUN, xreg=NULL, lambda=NULL, model=N
 #' can be arbitrarily small; if used for prediction interval calculations, they
 #' could lead to misleadingly small values.
 #'
-#' @param object An object of class "\code{modelAR}" resulting from a call to
-#' \code{\link{modelAR}}.
-#' @param h Number of periods for forecasting. If \code{xreg} is used, \code{h}
-#' is ignored and the number of forecast periods is set to the number of rows
-#' of \code{xreg}.
-#' @param PI If TRUE, prediction intervals are produced, otherwise only point
-#' forecasts are calculated. If \code{PI} is FALSE, then \code{level},
-#' \code{fan}, \code{bootstrap} and \code{npaths} are all ignored.
-#' @param level Confidence level for prediction intervals.
-#' @param fan If \code{TRUE}, level is set to \code{seq(51,99,by=3)}. This is
-#' suitable for fan plots.
-#' @param xreg Future values of external regressor variables.
-#' @param bootstrap If \code{TRUE}, then prediction intervals computed using
-#' simulations with resampled residuals rather than normally distributed
-#' errors. Ignored if \code{innov} is not \code{NULL}.
-#' @param npaths Number of sample paths used in computing simulated prediction
-#' intervals.
-#' @param innov Values to use as innovations for prediction intervals. Must be
-#' a matrix with \code{h} rows and \code{npaths} columns (vectors are coerced
-#' into a matrix). If present, \code{bootstrap} is ignored.
-#' @param ... Additional arguments passed to \code{\link{simulate.nnetar}}
-#' @inheritParams forecast.ts
-#'
-#' @return An object of class "\code{forecast}".
-#'
-#' The function \code{summary} is used to obtain and print a summary of the
-#' results, while the function \code{plot} produces a plot of the forecasts and
-#' prediction intervals.
-#'
-#' The generic accessor functions \code{fitted.values} and \code{residuals}
-#' extract useful features of the value returned by \code{forecast.nnetar}.
-#'
-#' An object of class "\code{forecast}" is a list containing at least the
-#' following elements:
-#'   \item{model}{A list containing information about the fitted model}
-#'   \item{method}{The name of the forecasting method as a character string}
-#'   \item{mean}{Point forecasts as a time series}
-#'   \item{lower}{Lower limits for prediction intervals}
-#'   \item{upper}{Upper limits for prediction intervals}
-#'   \item{level}{The confidence values associated with the prediction intervals}
-#'   \item{x}{The original time series (either \code{object} itself or the time series
-#'            used to create the model stored as \code{object}).}
-#'   \item{xreg}{The external regressors used in fitting (if given).}
-#'   \item{residuals}{Residuals from the fitted model. That is x minus fitted values.}
-#'   \item{fitted}{Fitted values (one-step forecasts)}
-#'   \item{...}{Other arguments}
+#' @inheritParams forecast.nnetar
+#' @param object An object of class `modelAR` resulting from a call to
+#' [modelAR()].
 #'
+#' @return An object of class `forecast`.
+#' @inheritSection forecast.ts forecast class
 #' @author Rob J Hyndman and Gabriel Caceres
-#' @seealso \code{\link{nnetar}}.
+#' @seealso [nnetar()].
 #' @keywords ts
 #'
 #' @export
-forecast.modelAR <- function(object, h=ifelse(object$m > 1, 2 * object$m, 10), PI=FALSE, level=c(80, 95), fan=FALSE, xreg=NULL, lambda=object$lambda, bootstrap=FALSE, npaths=1000, innov=NULL, ...) {
+forecast.modelAR <- function(
+  object,
+  h = if (object$m > 1) 2 * object$m else 10,
+  PI = FALSE,
+  level = c(80, 95),
+  fan = FALSE,
+  xreg = NULL,
+  lambda = object$lambda,
+  bootstrap = FALSE,
+  innov = NULL,
+  npaths = 1000,
+  ...
+) {
   out <- object
   tspx <- tsp(out$x)
-  #
-  if (fan) {
-    level <- seq(51, 99, by = 3)
-  } else {
-    if (min(level) > 0 && max(level) < 1) {
-      level <- 100 * level
-    } else if (min(level) < 0 || max(level) > 99.99) {
-      stop("Confidence limit out of range")
-    }
-  }
+  level <- getConfLevel(level, fan)
   # Check if xreg was used in fitted model
   if (is.null(object$xreg)) {
     if (!is.null(xreg)) {
-      warning("External regressors were not used in fitted model, xreg will be ignored")
+      warning(
+        "External regressors were not used in fitted model, xreg will be ignored"
+      )
     }
     xreg <- NULL
-  }
-  else {
+  } else {
     if (is.null(xreg)) {
       stop("No external regressors provided")
     }
@@ -411,7 +419,11 @@ forecast.modelAR <- function(object, h=ifelse(object$m > 1, 2 * object$m, 10), P
   if (!is.null(object$scalex)) {
     xx <- scale(xx, center = object$scalex$center, scale = object$scalex$scale)
     if (!is.null(xreg)) {
-      xxreg <- scale(xreg, center = object$scalexreg$center, scale = object$scalexreg$scale)
+      xxreg <- scale(
+        xreg,
+        center = object$scalexreg$center,
+        scale = object$scalexreg$scale
+      )
     }
   }
 
@@ -420,11 +432,12 @@ forecast.modelAR <- function(object, h=ifelse(object$m > 1, 2 * object$m, 10), P
   maxlag <- max(lags)
   flag <- rev(tail(xx, n = maxlag))
   # Iterative 1-step forecast
-  for (i in 1:h)
-  {
+  for (i in 1:h) {
     newdata <- c(flag[lags], xxreg[i, ])
-    if (any(is.na(newdata))) {
-      stop("I can't forecast when there are missing values near the end of the series.")
+    if (anyNA(newdata)) {
+      stop(
+        "I can't forecast when there are missing values near the end of the series."
+      )
     }
     fcast[i] <- object$predict.FUN(object$model, newdata)
     flag <- c(fcast[i], flag[-maxlag])
@@ -442,29 +455,20 @@ forecast.modelAR <- function(object, h=ifelse(object$m > 1, 2 * object$m, 10), P
   # Compute prediction intervals using simulations
   if (isTRUE(PI)) {
     nint <- length(level)
-    sim <- matrix(NA, nrow = npaths, ncol = h)
-    if (!is.null(innov)) {
-      if (length(innov) != h * npaths) {
-        stop("Incorrect number of innovations, need h*npaths values")
-      }
-      innov <- matrix(innov, nrow = h, ncol = npaths)
-      bootstrap <- FALSE
-    }
-    for (i in 1:npaths)
-      sim[i, ] <- simulate(object, nsim = h, bootstrap = bootstrap, xreg = xreg, lambda = lambda, innov = innov[, i], ...)
-    lower <- apply(sim, 2, quantile, 0.5 - level / 200, type = 8)
-    upper <- apply(sim, 2, quantile, 0.5 + level / 200, type = 8)
-    if (nint > 1L) {
-      lower <- ts(t(lower))
-      upper <- ts(t(upper))
-    }
-    else {
-      lower <- ts(matrix(lower, ncol = 1L))
-      upper <- ts(matrix(upper, ncol = 1L))
-    }
+    hilo <- simulate_forecast(
+      object = object,
+      h = h,
+      level = level,
+      npaths = npaths,
+      bootstrap = bootstrap,
+      innov = innov,
+      lambda = lambda,
+      ...
+    )
+    lower <- ts(hilo$lower)
+    upper <- ts(hilo$upper)
     tsp(lower) <- tsp(upper) <- tsp(fcast)
-  }
-  else {
+  } else {
     level <- NULL
     lower <- NULL
     upper <- NULL
@@ -473,17 +477,16 @@ forecast.modelAR <- function(object, h=ifelse(object$m > 1, 2 * object$m, 10), P
   out$level <- level
   out$lower <- lower
   out$upper <- upper
-  return(structure(out, class = "forecast"))
+  structure(out, class = "forecast")
 }
 
 #' @rdname fitted.Arima
 #' @export
-fitted.modelAR <- function(object, h=1, ...) {
+fitted.modelAR <- function(object, h = 1, ...) {
   if (h == 1) {
-    return(object$fitted)
-  }
-  else {
-    return(hfitted(object = object, h = h, FUN = "modelAR", ...))
+    object$fitted
+  } else {
+    hfitted(object = object, h = h, FUN = "modelAR", ...)
   }
 }
 
@@ -493,10 +496,11 @@ print.modelAR <- function(x, digits = max(3, getOption("digits") - 3), ...) {
   cat("Model: ", x$method, "\n")
   cat("Call:   ")
   print(x$call)
-  print(x$model)
   cat(
-    "\nsigma^2 estimated as ", format(mean(residuals(x) ^ 2, na.rm = TRUE), digits = digits),
-    "\n", sep = ""
+    "sigma^2 estimated as ",
+    format(mean(residuals(x)^2, na.rm = TRUE), digits = digits),
+    "\n",
+    sep = ""
   )
   invisible(x)
 }
@@ -506,3 +510,22 @@ print.modelAR <- function(x, digits = max(3, getOption("digits") - 3), ...) {
 is.modelAR <- function(x) {
   inherits(x, "modelAR")
 }
+
+#' @export
+residuals.modelAR <- function(
+  object,
+  type = c("innovation", "response"),
+  h = 1,
+  ...
+) {
+  y <- getResponse(object)
+  type <- match.arg(type)
+  if (type == "innovation" && !is.null(object$lambda)) {
+    res <- object$residuals
+  } else {
+    res <- y - fitted(object, h = h)
+  }
+  res <- ts(res)
+  tsp(res) <- tsp(y)
+  res
+}
